@@ -96,6 +96,10 @@
 #define V4L2_PIX_FMT_OV518 v4l2_fourcc('O', '5', '1', '8') /* ov518 JPEG */
 #endif
 
+#ifndef V4L2_PIX_FMT_STV0680
+#define V4L2_PIX_FMT_STV0680 v4l2_fourcc('S', '6', '8', '0')
+#endif
+
 #ifndef V4L2_FMT_FLAG_EMULATED
 #define V4L2_FMT_FLAG_EMULATED 0x0002
 #endif
@@ -233,6 +237,9 @@ void v4lconvert_decode_sn9c2028(const unsigned char *src, unsigned char *dst,
 void v4lconvert_decode_sq905c(const unsigned char *src, unsigned char *dst,
   int width, int height);
 
+void v4lconvert_decode_stv0680(const unsigned char *src, unsigned char *dst,
+  int width, int height);
+
 void v4lconvert_bayer_to_rgb24(const unsigned char *bayer,
   unsigned char *rgb, int width, int height, unsigned int pixfmt);
 
